// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef KUNIFIEDPUSH_VERSION_H
#define KUNIFIEDPUSH_VERSION_H

#define KUNIFIEDPUSH_VERSION_STRING "26.08.0"
#define KUNIFIEDPUSH_VERSION_MAJOR 26
#define KUNIFIEDPUSH_VERSION_MINOR 8
#define KUNIFIEDPUSH_VERSION_PATCH 0
#define KUNIFIEDPUSH_VERSION ((26<<16)|(8<<8)|(0))

#endif
