
#ifndef KUNIFIEDPUSH_EXPORT_H
#define KUNIFIEDPUSH_EXPORT_H

#ifdef KUNIFIEDPUSH_STATIC_DEFINE
#  define KUNIFIEDPUSH_EXPORT
#  define KUNIFIEDPUSH_NO_EXPORT
#else
#  ifndef KUNIFIEDPUSH_EXPORT
#    ifdef KUnifiedPush_EXPORTS
        /* We are building this library */
#      define KUNIFIEDPUSH_EXPORT __attribute__((visibility("default")))
#    else
        /* We are using this library */
#      define KUNIFIEDPUSH_EXPORT __attribute__((visibility("default")))
#    endif
#  endif

#  ifndef KUNIFIEDPUSH_NO_EXPORT
#    define KUNIFIEDPUSH_NO_EXPORT __attribute__((visibility("hidden")))
#  endif
#endif

#ifndef KUNIFIEDPUSH_DEPRECATED
#  define KUNIFIEDPUSH_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KUNIFIEDPUSH_DEPRECATED_EXPORT
#  define KUNIFIEDPUSH_DEPRECATED_EXPORT KUNIFIEDPUSH_EXPORT KUNIFIEDPUSH_DEPRECATED
#endif

#ifndef KUNIFIEDPUSH_DEPRECATED_NO_EXPORT
#  define KUNIFIEDPUSH_DEPRECATED_NO_EXPORT KUNIFIEDPUSH_NO_EXPORT KUNIFIEDPUSH_DEPRECATED
#endif

/* NOLINTNEXTLINE(readability-avoid-unconditional-preprocessor-if) */
#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KUNIFIEDPUSH_NO_DEPRECATED
#    define KUNIFIEDPUSH_NO_DEPRECATED
#  endif
#endif

#endif /* KUNIFIEDPUSH_EXPORT_H */
